@@ -2,6 +2,7 @@
 /*
  *  MICO --- an Open Source CORBA implementation
  *  Copyright (C) 1998 Frank Pilhofer
+ *  Copyright (c) 1999-2008 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,6 +28,10 @@
 #include <CORBA.h>
 #include <mico/impl.h>
 
+#ifdef USE_CSL2
+#include <mico/security/odm_impl.h>
+#endif /* USE_CSL2  */
+
 /*
  * Template definition for Policy objects must be outside
  * a "struct" (namespace)
@@ -42,7 +47,7 @@ private:
 
 public:
   POA_Policy (CORBA::PolicyType t, V val)
-      : MICO_SCOPE(MICO,Policy_impl) (t), _value (val) {};
+      : MICO::Policy_impl(t), _value (val) {};
   V value () { return _value; };
 
   CORBA::Policy_ptr copy ()
@@ -59,9 +64,9 @@ public:
  * ----------------------------------------------------------------------
  */
 
-MICO_NAMESPACE_DECL MICOPOA {
+namespace MICOPOA {
 
-MICO_EXPORT_DECL void _init ();
+extern void _init ();
 
 class POA_impl;
 class POACurrent_impl;
@@ -73,14 +78,14 @@ class POACurrent_impl;
 
 class POAOptions {
 public:
-  CORBA::Boolean parse (CORBA::ORB_ptr, int &, char *[]);
+  CORBA::Boolean parse (CORBA::ORB_ptr, const std::vector<std::string>&);
   const char * operator[] (const char *);
   
 private:
-  map<string,string,less<string> > options;
+  std::map<std::string, std::string, std::less<std::string> > options;
 };
 
-MICO_EXPORT_VAR_DECL POAOptions poaopts;
+extern MICO_EXPORT POAOptions poaopts;
 
 /*
  * ----------------------------------------------------------------------
@@ -109,8 +114,14 @@ typedef POA_Policy<PortableServer::RequestProcessingPolicy, PortableServer::Requ
 class POAManager_impl : virtual public PortableServer::POAManager
 {
 private:
+//    typedef vector<PortableServer::POA_ptr> POAPtrVec;
+
   State _state;
-  vector<PortableServer::POA_ptr> managed;
+
+  std::vector<PortableServer::POA_ptr> managed;
+
+  MICOMT::Mutex managed_lock;
+
   void change_state (State,
 		     CORBA::Boolean = FALSE,
 		     CORBA::Boolean = FALSE);
@@ -212,6 +223,8 @@ public:
   char * next_descendant_poa (const char *, const char *);
   
 private:
+  MICOMT::Mutex _ref_lock;
+
   void make_ref ();
   bool decompose_ref ();
 
@@ -226,8 +239,8 @@ private:
    */
 
   bool iddirty;
-  string poaname;
-  string repoid;
+  std::string poaname;
+  std::string repoid;
   MICOPOA::ObjectId oid;
   CORBA::Object_ptr obj;
   PortableServer::Servant servant;
@@ -242,15 +255,30 @@ private:
 
 class ObjectMap {
 public:
+  struct DeletionRecord {
+    CORBA::Long  cnt;
+    CORBA::Short signal;
+    MICOMT::CondVar condition;
+
+    DeletionRecord(MICOMT::Mutex* m)
+      : cnt(0), signal(0), condition(m)
+    {}
+  };
+
   struct ObjectRecord {
     ObjectRecord (POAObjectReference *,
 		  PortableServer::Servant = NULL);
     ~ObjectRecord ();
+
+    CORBA::Boolean active;
+    CORBA::Long invoke_cnt;
+    DeletionRecord * delref;
+
     POAObjectReference * por;
     PortableServer::Servant serv;
   };
 
-  typedef map<ObjectId, ObjectRecord *, less<ObjectId> > IdMap;
+  typedef std::map<ObjectId, ObjectRecord *, std::less<ObjectId> > IdMap;
   typedef IdMap::iterator iterator;
 
 public:
@@ -264,7 +292,7 @@ public:
 
   ObjectRecord * pop ();
 
-  void add (POAObjectReference *, PortableServer::Servant);
+  ObjectRecord * add (POAObjectReference *, PortableServer::Servant);
   ObjectRecord * del (const ObjectId &);
   ObjectRecord * del (const PortableServer::ObjectId &);
 
@@ -280,8 +308,8 @@ public:
   ObjectRecord * find (PortableServer::Servant);
 
 private:
-  typedef map<PortableServer::Servant, vector<ObjectRecord *>,
-    less<PortableServer::Servant> > SvMap;
+  typedef std::map<PortableServer::Servant, std::vector<ObjectRecord *>,
+    std::less<PortableServer::Servant> > SvMap;
 
   IdMap objs;
   SvMap servants;
@@ -334,7 +362,7 @@ public:
   
   class InvocationRecord : public CORBA::ServerlessObject {
   public:
-    InvocationRecord (CORBA::ORB::MsgId,
+    InvocationRecord (CORBA::ORBMsgId,
 		      POAObjectReference *,
 		      CORBA::ORBRequest *,
 		      CORBA::Principal_ptr);
@@ -342,7 +370,7 @@ public:
 
     void exec (POA_impl *);
 
-    CORBA::ORB::MsgId id ();
+    CORBA::ORBMsgId id ();
     CORBA::ORBRequest * get_or ();
     POAObjectReference * get_por ();
     CORBA::ServerRequestBase_ptr make_req (POA_impl *,
@@ -361,7 +389,7 @@ public:
     }
 
   private:
-    CORBA::ORB::MsgId msgid;
+    CORBA::ORBMsgId orbid;
     POAObjectReference * por;
     CORBA::ORBRequest * req;
     CORBA::Principal_ptr pr;
@@ -385,22 +413,23 @@ private:
    * POA Attributes
    */
 
-  string name;            // name relative to parent
-  string fqn;             // fully qualified name
-  string oaid;            // Unique Identifier
-  static string oaprefix; // Prefix for Unique Names
+  std::string name;            // name relative to parent
+  std::string fqn;             // fully qualified name
+  std::string oaid;            // Unique Identifier
+  static std::string oaprefix; // Prefix for Unique Names
 
   POA_impl * parent;
   PortableServer::POAManager_ptr manager;
   PortableServer::Servant default_servant;
   PortableServer::ServantManager_var servant_manager;
+  static MICOMT::Mutex S_servant_manager_lock;
   PortableServer::AdapterActivator_var adapter_activator;
 
   /*
    * POAMediator Stuff
    */
 
-  static string impl_name;
+  static std::string impl_name;
   static CORBA::IOR poamed_ior;
   static CORBA::POAMediator_var poamed;
   static CORBA::Boolean ever_been_active;
@@ -410,17 +439,28 @@ private:
    */
 
   int destructed;
+  MICOMT::RWLock destroy_lock_;
   CORBA::ULong unique_id;
   CORBA::ORB_ptr orb;
+  CORBA::IOR* ior_template_;
+
   ObjectMap ActiveObjectMap;
-  vector<InvocationRecord_ptr> InvocationQueue;
+  MICOMT::Mutex ObjectActivationLock;
+
+  std::vector<InvocationRecord_ptr> InvocationQueue;
   PortableServer::POAManager::State state;
 
+  /*
+   * serialization
+   */
+  MICOMT::Mutex serialize_invoke;
+  static MICOMT::Mutex S_global_invoke_lock;
+
   /*
    * Map of our children
    */
 
-  typedef map<string, POA_impl *, less<string> > POAMap;
+  typedef std::map<std::string, POA_impl *, std::less<std::string> > POAMap;
 
   void register_child (const char *, POA_impl *);
   void unregister_child (const char *);
@@ -444,6 +484,15 @@ private:
 
   static POACurrent_impl * current;
 
+#ifdef USE_CSL2
+#ifdef HAVE_SSL // ###ras
+  // ODM Manager
+  MICOSODM::Manager_impl* odm_manager_;
+  // ODM Factory
+  MICOSODM::Factory_impl* odm_factory_;
+#endif // HAVE_SSL
+#endif // USE_CSL2
+
   /*
    * private ops
    */
@@ -463,6 +512,13 @@ private:
 	    POA_impl *,
 	    CORBA::ORB_ptr);
 
+  
+  /*
+   * private object activation and deactivation
+   */
+  PortableServer::ObjectId * __activate_object (PortableServer::Servant);
+  void remove_object (const PortableServer::ObjectId & id);
+
 public:
   POA_impl (CORBA::ORB_ptr);
   virtual ~POA_impl ();
@@ -488,6 +544,7 @@ public:
   PortableServer::AdapterActivator_ptr the_activator ();
   PortableServer::POAList * the_children ();
   void the_activator (PortableServer::AdapterActivator_ptr);
+  CORBA::OctetSeq* id();
 
   /*
    * Factories for Policy objects
@@ -578,24 +635,40 @@ public:
   CORBA::Boolean has_object (CORBA::Object_ptr);
   CORBA::Boolean is_local   () const;
 
-  CORBA::Boolean invoke (CORBA::ORB::MsgId,
+#ifdef USE_CSL2
+  CORBA::Principal_ptr get_principal (CORBA::Object_ptr);
+#endif /* USE_CSL2  */
+
+  CORBA::Boolean invoke (CORBA::ORBMsgId,
 			 CORBA::Object_ptr,
 			 CORBA::ORBRequest *,
 			 CORBA::Principal_ptr,
 			 CORBA::Boolean = TRUE);
-  CORBA::Boolean bind   (CORBA::ORB::MsgId, const char *,
+  CORBA::Boolean bind   (CORBA::ORBMsgId, const char *,
 			 const CORBA::ORB::ObjectTag &,
 			 CORBA::Address *);
-  CORBA::Boolean locate (CORBA::ORB::MsgId, CORBA::Object_ptr);
+  CORBA::Boolean locate (CORBA::ORBMsgId, CORBA::Object_ptr);
   CORBA::Object_ptr skeleton (CORBA::Object_ptr);
 
-  void cancel   (MsgId);
+  void cancel   (CORBA::ORBMsgId);
   void shutdown (CORBA::Boolean);
 
-  void answer_invoke (CORBA::ORB::MsgId, CORBA::Object_ptr,
+  void answer_invoke (CORBA::ORBMsgId, CORBA::Object_ptr,
 		      CORBA::ORBRequest *,
 		      CORBA::InvokeStatus);
 
+  virtual CORBA::Boolean
+  validate_connection(CORBA::Object_ptr, CORBA::PolicyList_out);
+
+  virtual void
+  timedout_invoke(CORBA::ORBMsgId);
+
+#ifdef USE_CSL2
+  // ODM stuff ###ras
+  virtual void register_ODM_factory(ObjectDomainMapping::Factory_ptr fry);
+  virtual ObjectDomainMapping::Manager_ptr get_ODM();
+#endif /* USE_CSL2  */
+
   /*
    * Helper functions for OA interface
    */
@@ -621,24 +694,6 @@ public:
 
 class POACurrent_impl : public PortableServer::Current
 {
-public:
-  POACurrent_impl (CORBA::ORB_ptr);
-  ~POACurrent_impl ();
-
-  PortableServer::POA_ptr get_POA ();
-  PortableServer::ObjectId * get_object_id ();
-
-  // begin-mico-extension
-  CORBA::Boolean iscurrent ();
-  CORBA::Object_ptr make_ref ();
-  PortableServer::Servant get_serv ();
-  POAObjectReference * get_por ();
-
-  void set (PortableServer::POA_ptr, POAObjectReference *,
-	    PortableServer::Servant);
-  void unset ();
-  // end-mico-extension
-
 private:
   CORBA::ORB_ptr orb;
 
@@ -654,9 +709,58 @@ private:
     PortableServer::Servant serv;
   };
 
-  vector<CurrentState> CurrentStateStack;
+public:
+  typedef std::vector<CurrentState> CurrentStateStack;
+
+private:
+#ifndef HAVE_THREADS
+  CurrentStateStack* state_stack_;
+#else // HAVE_THREADS
+  MICOMT::Thread::ThreadKey current_key_;
+#endif // HAVE_THREADS
+
+public:
+  POACurrent_impl (CORBA::ORB_ptr);
+  ~POACurrent_impl ();
+
+  PortableServer::POA_ptr get_POA ();
+  PortableServer::ObjectId * get_object_id ();
+  CORBA::Object_ptr get_reference();
+  PortableServer::Servant get_servant();
+  // begin-mico-extension
+  CORBA::Boolean iscurrent ();
+  POAObjectReference * get_por ();
+
+  void set (PortableServer::POA_ptr, POAObjectReference *,
+	    PortableServer::Servant);
+  void unset ();
+
+#ifndef HAVE_THREADS
+  //
+  // In single thread mode, only one POA can be active at any given time
+  // a stack of POA/Currents is OK here, because nested invocation are not possible 
+  //
+  CurrentStateStack* get_current()
+  { return state_stack_; }
+
+  void set_current(CurrentStateStack* stack)
+  { state_stack_ = stack; }
+
+#else // HAVE_THREADS
+  //
+  // In multi threaded mode, more than one POA can be active (doing invocations),
+  // but there can be only one active Object per thread 
+  // 
+  CurrentStateStack* get_current()
+  { return static_cast<CurrentStateStack *>(MICOMT::Thread::get_specific(current_key_)); }
+
+  void set_current(CurrentStateStack* current)
+  { MICOMT::Thread::set_specific(current_key_, current); }
+
+#endif // HAVE_THREADS
+  // end-mico-extension
 };
 
-} MICO_NAMESPACE_END_DECL // namespace MICOPOA
+}  // namespace MICOPOA
 
 #endif
