@@ -46,7 +46,7 @@ public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
     typedef TSeqVar<StringSequenceTmpl<T> > _var_type;
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     StringSequenceTmpl () {}
     StringSequenceTmpl (MICO_ULong maxval)
@@ -115,7 +115,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const StringSequenceTmpl<T> &v1,
 		const StringSequenceTmpl<T> &v2)
@@ -200,7 +200,7 @@ StringSequenceTmpl<T>::operator[] (MICO_ULong idx) const
     return vec[idx];
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T>
 MICO_Boolean
 operator== (const StringSequenceTmpl<T> &v1, const StringSequenceTmpl<T> &v2)
@@ -224,7 +224,7 @@ class BoundedStringSequenceTmpl {
 public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     BoundedStringSequenceTmpl ()
     {
@@ -304,7 +304,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const BoundedStringSequenceTmpl<T,max> &v1,
 		const BoundedStringSequenceTmpl<T,max> &v2)
@@ -367,7 +367,7 @@ BoundedStringSequenceTmpl<T,max>::length (MICO_ULong l)
     }
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T, int max>
 MICO_Boolean
 operator== (const BoundedStringSequenceTmpl<T,max> &v1,
@@ -392,7 +392,7 @@ public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
     typedef TSeqVar<WStringSequenceTmpl<T> > _var_type;
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     WStringSequenceTmpl () {}
     WStringSequenceTmpl (MICO_ULong maxval)
@@ -461,7 +461,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const WStringSequenceTmpl<T> &v1,
 		const WStringSequenceTmpl<T> &v2)
@@ -546,7 +546,7 @@ WStringSequenceTmpl<T>::operator[] (MICO_ULong idx) const
     return vec[idx];
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T>
 MICO_Boolean
 operator== (const WStringSequenceTmpl<T> &v1, const WStringSequenceTmpl<T> &v2)
@@ -570,7 +570,7 @@ class BoundedWStringSequenceTmpl {
 public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     BoundedWStringSequenceTmpl ()
     {
@@ -650,7 +650,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const BoundedWStringSequenceTmpl<T,max> &v1,
 		const BoundedWStringSequenceTmpl<T,max> &v2)
@@ -713,7 +713,7 @@ BoundedWStringSequenceTmpl<T,max>::length (MICO_ULong l)
     }
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T, int max>
 MICO_Boolean
 operator== (const BoundedWStringSequenceTmpl<T,max> &v1,
@@ -738,7 +738,7 @@ public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
     typedef TSeqVar<IfaceSequenceTmpl<T,T_ptr> > _var_type;
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     IfaceSequenceTmpl () {}
     IfaceSequenceTmpl (MICO_ULong maxval)
@@ -807,7 +807,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const IfaceSequenceTmpl<T,T_ptr> &v1,
 		const IfaceSequenceTmpl<T,T_ptr> &v2)
@@ -894,7 +894,7 @@ IfaceSequenceTmpl<T,T_ptr>::operator[] (MICO_ULong idx) const
     return vec[idx];
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T, class T_ptr>
 MICO_Boolean
 operator== (const IfaceSequenceTmpl<T,T_ptr> &v1,
@@ -919,7 +919,7 @@ class BoundedIfaceSequenceTmpl {
 public:
     typedef T &ElementType; // Needed in TSeqVar (see var.h)
 private:
-    vector<T> vec;
+    std::vector<T> vec;
 public:
     BoundedIfaceSequenceTmpl ()
     {
@@ -999,7 +999,7 @@ public:
 	delete[] b;
     }
 
-#if defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200)
+#if defined( __SUNPRO_CC )
     friend MICO_Boolean
     operator== (const BoundedIfaceSequenceTmpl<T,T_ptr,max> &v1,
 		const BoundedIfaceSequenceTmpl<T,T_ptr,max> &v2)
@@ -1065,7 +1065,7 @@ BoundedIfaceSequenceTmpl<T,T_ptr,max>::length (MICO_ULong l)
     }
 }
 
-#if !(defined( __SUNPRO_CC ) || (defined( _WINDOWS ) && _MSC_VER < 1200))
+#if !defined( __SUNPRO_CC )
 template<class T, class T_ptr, int max>
 MICO_Boolean
 operator== (const BoundedIfaceSequenceTmpl<T,T_ptr,max> &v1,
