@@ -28,8 +28,8 @@ namespace MICO {
 
 class UnixSharedLib : public MICO::SharedLib {
     void *_handle;
-    string _error;
-    string _name;
+    std::string _error;
+    std::string _name;
 public:
     UnixSharedLib (const char *name);
     virtual ~UnixSharedLib ();
