@@ -1,8 +1,12 @@
 
-//#define USE_POA
+#define USE_POA
 
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include "bench.h"
 #include "bench_impl.h"
 
