@@ -1,10 +1,14 @@
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include "hello.h"
 
 static const char *serverid;
 
-class Hello_impl : virtual public POA_Hello {
+class Hello_impl : virtual public POA_HelloWorld {
 public:
     char *hello ()
     {
