@@ -1,5 +1,8 @@
 #include "custom_impl.h"
 
+
+using namespace std;
+
 CORBA::Long
 CustomValue_impl::value ()
 {
@@ -15,7 +18,7 @@ CustomValue_impl::marshal (CORBA::DataOutputStream * os)
   os->write_string ("Hello World");
 }
 
-CORBA::ValueBase *
+void
 CustomValue_impl::unmarshal (CORBA::DataInputStream * is)
 {
   cout << "custom unmarshaller" << endl;
@@ -23,6 +26,5 @@ CustomValue_impl::unmarshal (CORBA::DataInputStream * is)
   CORBA::Short s = is->read_short ();
   CORBA::String_var v = is->read_string ();
   assert (b == TRUE && s == 42 && strcmp (v.in(), "Hello World") == 0);
-  return this;
 }
 
