@@ -11,7 +11,7 @@ public:
   CORBA::Long value ();
 
   void marshal (CORBA::DataOutputStream *os);
-  CORBA::ValueBase * unmarshal (CORBA::DataInputStream *is);
+  void unmarshal (CORBA::DataInputStream *is);
 };
 
 class CustomValue_Factory : virtual public CORBA::ValueFactoryBase
