@@ -6,6 +6,9 @@
 #include "tricky_impl.h"
 #include <string.h>
 
+
+using namespace std;
+
 class iface_impl : virtual public POA_iface
 {
 public:
@@ -52,7 +55,7 @@ main (int argc, char *argv[])
   PortableServer::ServantBase_var iface = new iface_impl;
   CORBA::Object_var obj = iface->_this ();
 
-  printf ("Running.\n");
+  cout << "Running." << endl;
   mgr->activate ();
   orb->run ();
 
