@@ -10,10 +10,13 @@
 #include <unistd.h>
 #endif
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 #include <direct.h>
 #endif
 
+
+using namespace std;
+
 int
 main (int argc, char *argv[])
 {
@@ -34,7 +37,7 @@ main (int argc, char *argv[])
   Bank_var bank = Bank::_narrow (obj);
 
   if (CORBA::is_nil (bank)) {
-    printf ("oops: could not locate Bank\n");
+    cout << "oops: could not locate Bank" << endl;
     exit (1);
   }
 
@@ -45,7 +48,7 @@ main (int argc, char *argv[])
   Account_var account = bank->create ("Frank");
 
   if (CORBA::is_nil (account)) {
-    printf ("oops: account is nil\n");
+    cout << "oops: account is nil" << endl;
     exit (1);
   }
 
@@ -61,8 +64,10 @@ main (int argc, char *argv[])
   account->withdraw (450);
   bank->shutdown ();
 
-  printf ("Balance is %ld.\n", account->balance ());
+  cout << "Balance is " << account->balance() << endl;
   bank->shutdown ();
 
+  orb->shutdown(TRUE);
+  //orb->destroy();
   return 0;
 }
