@@ -2,8 +2,15 @@
  * A simple "Hello World" example that uses the POA
  */
 
-#include <fstream.h>
 #include "hello.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <fstream>
+#else
+#include <fstream.h>
+#endif
+
+
+using namespace std;
 
 /*
  * Hello World implementation. This implementation does not inherit
@@ -19,7 +26,7 @@ public:
 void
 HelloWorld_impl::hello ()
 {
-  printf ("Hello World\n");
+  cout << "Hello World" << endl;
 }
 
 int
@@ -66,7 +73,7 @@ main (int argc, char *argv[])
    * Activate the POA and start serving requests
    */
 
-  printf ("Running.\n");
+  cout << "Running." << endl;
 
   mgr->activate ();
   orb->run();
