@@ -1,13 +1,19 @@
-#include <iostream.h>
 #include <CORBA-SMALL.h>
-#include <mico/CosNaming.h>
-#include <mico/CosEventComm.h>
-#include <mico/CosEventChannelAdmin.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
+#include <iostream.h>
+#endif
+#include <coss/CosNaming.h>
+#include <coss/CosEventComm.h>
+#include <coss/CosEventChannelAdmin.h>
+
+
+using namespace std;
 
 int main (int argc, char** argv)
 {
   CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
-  CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");
 
   CORBA::Object_var nsobj =
     orb->resolve_initial_references ("NameService");
