@@ -1,7 +1,7 @@
 // sound.h
 #include <mico/stream.h>
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 #include <fstream>
 #include <iostream>
 #else
