@@ -31,12 +31,12 @@ main (int argc, char *argv[])
     CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");
 
     assert (argc == 2);
-    CORBA::Object_var obj = orb->bind ("IDL:Hello:1.0", argv[1]);
+    CORBA::Object_var obj = orb->bind ("IDL:HelloWorld:1.0", argv[1]);
     if (CORBA::is_nil (obj)) {
 	cerr << "cannot bind to " << argv[1] << endl;
 	return 1;
     }
-    Hello_var h = Hello::_narrow (obj);
+    HelloWorld_var h = HelloWorld::_narrow (obj);
     h->hello ("hello world");
     return 0;
 }
