@@ -25,7 +25,7 @@
 #include <iostream.h>
 #include "hello.h"
 
-class Hello_impl : virtual public Hello_skel {
+class HelloWorld_impl : virtual public HelloWorld_skel {
 public:
     void hello (const char *s)
     {
@@ -33,14 +33,14 @@ public:
     }
 };
 
-static Hello_ptr server = Hello::_nil();
+static HelloWorld_ptr server = HelloWorld::_nil();
 
 extern "C" CORBA::Boolean
 mico_module_init (const char *version)
 {
     if (strcmp (version, MICO_VERSION))
 	return FALSE;
-    server = new Hello_impl;
+    server = new HelloWorld_impl;
     return TRUE;
 }
 
