@@ -20,10 +20,11 @@
  *                 mico@informatik.uni-frankfurt.de
  */
 
-#include <iostream.h>
-#include <sys/time.h>
 #include "hello.h"
 
+
+using namespace std;
+
 int
 main (int argc, char *argv[])
 {
@@ -32,7 +33,7 @@ main (int argc, char *argv[])
     assert (argc == 2);
     CORBA::Object_var obj = orb->string_to_object (argv[1]);
     if (CORBA::is_nil (obj)) {
-	cerr << "cannot bind to " << argv[1] << endl;
+	cout << "cannot bind to " << argv[1] << endl;
 	return 1;
     }
     SecureHello_var hello = SecureHello::_narrow (obj);
